import 'package:dio/dio.dart';
import 'package:flutter/foundation.dart';
import 'package:n_bug_reporter/model/clickup_task.dart';
import 'package:n_bug_reporter/model/clickup_task_attachment.dart';
import 'package:n_bug_reporter/model/create_task_template_body.dart';
import 'dart:typed_data';

class DioClient {
  static const String _clickUpBaseUrl = 'https://api.clickup.com/api/v2';
  final Dio _dio = Dio(BaseOptions(baseUrl: DioClient._clickUpBaseUrl));

  setAuthorizationHeader(String authorizationHeader) {
    _dio.options.headers['Authorization'] = authorizationHeader;
  }

  // for cors error: https://stackoverflow.com/questions/65630743/how-to-solve-flutter-web-api-cors-error-only-with-dart-code/66879350#66879350
  Future<ClickUpTask?> createTaskFromTemplate(
      {required String listId,
      required String templateId,
      required CreateTaskTemplateBody body}) async {
    ClickUpTask? task;
    try {
      Response taskData = await _dio
          .post('/list/$listId/taskTemplate/$templateId', data: body.toJson());
      if (kDebugMode) {
        print('Task Info: ${taskData.data}');
      }
      task = ClickUpTask.fromJson(taskData.data);
    } on DioError catch (e) {
      if (kDebugMode) {
        if (e.response != null) {
          print('Dio error!');
          print('STATUS: ${e.response?.statusCode}');
          print('DATA: ${e.response?.data}');
          print('HEADERS: ${e.response?.headers}');
        } else {
          print('Error sending request!');
          print(e.message);
        }
      } else {
        rethrow;
      }
    }
    return task;
  }

  Future setCustomFieldForTask(
      {required String taskId,
      required String fieldId,
      required String value}) async {
    try {
      Response customFieldData = await _dio
          .post('/task/$taskId/field/$fieldId', data: {'value': value});
      if (kDebugMode) {
        print('Custom Field Info: ${customFieldData.data}');
      }
    } on DioError catch (e) {
      if (kDebugMode) {
        if (e.response != null) {
          print('Dio error!');
          print('STATUS: ${e.response?.statusCode}');
          print('DATA: ${e.response?.data}');
          print('HEADERS: ${e.response?.headers}');
        } else {
          print('Error sending request!');
          print(e.message);
        }
      } else {
        rethrow;
      }
    }
  }

  Future<ClickUpTaskAttachment?> createTaskAttachment(
      {required Uint8List pngBytes,
      required String fileName,
      required String taskId}) async {
    var formData = FormData();
    formData.files.add(MapEntry(
        'attachment', MultipartFile.fromBytes(pngBytes, filename: fileName)));

    ClickUpTaskAttachment? taskAttachment;
    try {
      Response taskAttachmentData =
          await _dio.post('/task/$taskId/attachment', data: formData);
      if (kDebugMode) {
        print('Task Attachment Info: ${taskAttachmentData.data}');
      }
      taskAttachment = ClickUpTaskAttachment.fromJson(taskAttachmentData.data);
    } on DioError catch (e) {
      if (kDebugMode) {
        if (e.response != null) {
          print('Dio error!');
          print('STATUS: ${e.response?.statusCode}');
          print('DATA: ${e.response?.data}');
          print('HEADERS: ${e.response?.headers}');
        } else {
          print('Error sending request!');
          print(e.message);
        }
      } else {
        rethrow;
      }
    }
    return taskAttachment;
  }
}
